@@ -103,6 +103,7 @@ typedef struct {
   SDL_Surface *__mapperSurface;
 
   int __cdgColourTable[COLOUR_TABLE_SIZE];
+  int __justClearedColourIndex;
   int __cdgPresetColourIndex;
   int __cdgBorderColourIndex;
   int __cdgTransparentColour;
@@ -215,7 +216,7 @@ do_rewind(CdgPacketReader *self) {
 
   defaultColour = 0;
   memset(self->__cdgColourTable, defaultColour, sizeof(int) * COLOUR_TABLE_SIZE);
-
+  self->__justClearedColourIndex = -1;
   self->__cdgPresetColourIndex = -1;
   self->__cdgBorderColourIndex = -1;
 
@@ -479,8 +480,25 @@ __cdgMemoryPreset(CdgPacketReader *self, CdgPacket *packd) {
   Uint32 presetColour;
 
   colour = packd->data[0] & 0x0F;
-  /* Ignore repeat because this is a reliable data stream */
   /* repeat = packd->data[1] & 0x0F; */
+
+  /* The "repeat" flag is nonzero if this is a repeat of a
+     previously-appearing preset command.  (Often a CDG will
+     issue several copies of this command in case one gets
+     corrupted.) */
+  
+  /* We could ignore the entire command if repeat is nonzero, but
+     our data stream is not 100% reliable, since it might have
+     come from a bad rip.  So we should honor every preset
+     command; but we shouldn't waste CPU time clearing the screen
+     repeatedly, needlessly.  So we store a flag indicating the
+     last color that we just cleared to, and don't bother
+     clearing again if it hasn't changed. */
+
+  if (colour == self->__justClearedColourIndex) {
+    return;
+  }
+  self->__justClearedColourIndex = colour;
   
   /* Our new interpretation of CD+G Revealed is that memory preset
      commands should also change the border */
@@ -519,6 +537,10 @@ __cdgBorderPreset(CdgPacketReader *self, CdgPacket *packd) {
   Uint32 borderColour;
 
   colour = packd->data[0] & 0x0F;
+  if (colour == self->__cdgBorderColourIndex) {
+    return;
+  }
+        
   self->__cdgBorderColourIndex = colour;
 
   /* See __cdgMemoryPreset() for a description of what's going on.
@@ -791,10 +813,14 @@ __cdgTileBlockCommon(CdgPacketReader *self, CdgPacket *packd, int xor) {
       self->__cdgPixelColours[row_index + j][column_index + i] = new_col;      
     }
   }
+
+  /* Now the screen has some data on it, so a subsequent clear
+     should be respected. */
+  self->__justClearedColourIndex = -1;
 }
 
 /* The rest of the lines in this module are boilerplate Python
-   constructs to formally export the appropriate the classes and
+   constructs to formally export the appropriate classes and
    methods from this module. */
 
 static PyMethodDef CdgPacketReader_methods[] = {
